@@ -37,6 +37,17 @@
   #include "config.h"
 #endif
 
+#if HAVE_LIBFIU
+  #include <fiu.h>
+  #include <fiu-control.h>
+#endif
+
+/* Include cunit header if c unit testing support is enabled. */
+#ifdef HAVE_C_UNIT_TESTS
+    #include "CUnit/Basic.h"
+    #include "cunit_common.h"
+#endif
+
 #include <stdio.h>
 
 #if HAVE_SYS_TYPES_H
@@ -86,6 +97,8 @@
   #define fdopen        _fdopen
   #define close			_close
   #define write			_write
+  #define popen			_popen
+  #define pclose		_pclose
   #define O_WRONLY		_O_WRONLY
   #define O_RDONLY		_O_RDONLY
   #define O_RDWR		_O_RDWR
@@ -132,7 +145,8 @@ enum {
 #define FKO_DEFAULT_PORT    62201
 #define DEFAULT_NAT_PORT    55000
 #define MIN_HIGH_PORT       10000  /* sensible minimum for SPA dest port */
-#define MAX_PORT            65535
+#define ANY_PORT            0      /* used as a wildcard */
+#define ANY_PROTO           0      /* used as a wildcard */
 #define MAX_SERVER_STR_LEN  50
 #define MAX_ICMP_TYPE       40
 #define MAX_ICMP_CODE       15
@@ -143,6 +157,14 @@ enum {
 #define MAX_GPG_KEY_ID      128
 #define MAX_USERNAME_LEN    30
 
+/* Command line argument / argv handling
+*/
+#define MAX_CMDLINE_ARGS            30                  /*!< should be way more than enough */
+
+#if HAVE_LIBFIU
+  #define MAX_FAULT_TAG_LEN 128
+#endif
+
 /* Some convenience macros */
 
 /* Get the number of elements of an array
