@@ -47,6 +47,11 @@
   #define SUCCESS    1
 #endif
 
+#ifdef HAVE_C_UNIT_TESTS
+  #include "cunit_common.h"
+  DECLARE_TEST_SUITE(access, "Access test suite");
+#endif
+
 /* Add an access string entry
 */
 static int
@@ -749,7 +754,7 @@ free_acc_stanza_data(acc_stanza_t *acc)
         free(acc->source);
         free_acc_int_list(acc->source_list);
     }
-    
+
     if(acc->destination != NULL)
     {
         free(acc->destination);
@@ -1072,7 +1077,7 @@ set_acc_defaults(fko_srv_options_t *opts)
 /* Perform some sanity checks on an acc stanza data.
 */
 static int
-acc_data_is_valid(struct passwd *user_pw, acc_stanza_t * const acc)
+acc_data_is_valid(fko_srv_options_t *opts, struct passwd *user_pw, acc_stanza_t * const acc)
 {
     if(acc == NULL)
     {
@@ -1134,14 +1139,24 @@ acc_data_is_valid(struct passwd *user_pw, acc_stanza_t * const acc)
         }
     }
 
-    if((acc->force_snat == 1 || acc->force_masquerade == 1) && acc->force_nat == 0)
+#if defined(FIREWALL_FIREWALLD) || defined(FIREWALL_IPTABLES)
+    if((acc->force_snat == 1 || acc->force_masquerade == 1)
+            && acc->force_nat == 0)
     {
-        log_msg(LOG_ERR,
-                "[*] FORCE_SNAT/FORCE_MASQUERADE implies FORCE_NAT must also be used for access stanza source: '%s'",
-                acc->source
-        );
-        return(0);
+        if(acc->forward_all == 1)
+        {
+            add_acc_force_nat(opts, acc, "0.0.0.0 0");
+        }
+        else
+        {
+            log_msg(LOG_ERR,
+                    "[*] FORCE_SNAT/FORCE_MASQUERADE requires either FORCE_NAT or FORWARD_ALL: '%s'",
+                    acc->source
+            );
+            return(0);
+        }
     }
+#endif
 
     if(acc->require_source_address == 0)
     {
@@ -1278,7 +1293,7 @@ parse_access_file(fko_srv_options_t *opts)
              * stanza for the minimum required data.
             */
             if(curr_acc != NULL) {
-                if(!acc_data_is_valid(user_pw, curr_acc))
+                if(!acc_data_is_valid(opts, user_pw, curr_acc))
                 {
                     log_msg(LOG_ERR, "[*] Data error in access file: '%s'",
                         opts->config[CONF_ACCESS_FILE]);
@@ -1707,6 +1722,14 @@ parse_access_file(fko_srv_options_t *opts)
             add_acc_bool(&(curr_acc->force_masquerade), val);
             add_acc_bool(&(curr_acc->force_snat), val);
         }
+        else if(CONF_VAR_IS(var, "DISABLE_DNAT"))
+        {
+            add_acc_bool(&(curr_acc->disable_dnat), val);
+        }
+        else if(CONF_VAR_IS(var, "FORWARD_ALL"))
+        {
+            add_acc_bool(&(curr_acc->forward_all), val);
+        }
         else
         {
             log_msg(LOG_ERR,
@@ -1732,7 +1755,7 @@ parse_access_file(fko_srv_options_t *opts)
 
     /* Sanity check the last stanza
     */
-    if(!acc_data_is_valid(user_pw, curr_acc))
+    if(!acc_data_is_valid(opts, user_pw, curr_acc))
     {
         log_msg(LOG_ERR,
             "[*] Data error in access file: '%s'",
@@ -1941,6 +1964,8 @@ dump_access_list(const fko_srv_options_t *opts)
             "           FORCE_NAT (port):  %d\n"
             "            FORCE_SNAT (ip):  %s\n"
             "           FORCE_MASQUERADE:  %s\n"
+            "               DISABLE_DNAT:  %s\n"
+            "                FORWARD_ALL:  %s\n"
             "              ACCESS_EXPIRE:  %s"  /* asctime() adds a newline */
             "               GPG_HOME_DIR:  %s\n"
             "                    GPG_EXE:  %s\n"
@@ -1972,6 +1997,8 @@ dump_access_list(const fko_srv_options_t *opts)
             acc->force_nat ? acc->force_nat_port : 0,
             acc->force_snat ? acc->force_snat_ip : "<not set>",
             acc->force_masquerade ? "Yes" : "No",
+            acc->disable_dnat ? "Yes" : "No",
+            acc->forward_all ? "Yes" : "No",
             (acc->access_expire_time > 0) ? asctime(localtime(&acc->access_expire_time)) : "<not set>\n",
             (acc->gpg_home_dir == NULL) ? "<not set>" : acc->gpg_home_dir,
             (acc->gpg_exe == NULL) ? "<not set>" : acc->gpg_exe,
@@ -1991,5 +2018,35 @@ dump_access_list(const fko_srv_options_t *opts)
     fprintf(stdout, "\n");
     fflush(stdout);
 }
+#ifdef HAVE_C_UNIT_TESTS
+
+DECLARE_UTEST(compare_port_list, "check compare_port_list function")
+{
+    acc_port_list_t *in1_pl = NULL;
+    acc_port_list_t *in2_pl = NULL;
+    acc_port_list_t *acc_pl = NULL;
+
+    /* Match any test */	
+    free_acc_port_list(in1_pl);
+    free_acc_port_list(acc_pl);
+    add_port_list_ent(&in1_pl, "udp/6002");
+    add_port_list_ent(&in2_pl, "udp/6002, udp/6003");
+    add_port_list_ent(&acc_pl, "udp/6002, udp/6003");
+    CU_ASSERT(compare_port_list(in1_pl, acc_pl, 1) == 1);	/* Only one match is needed from access port list - 1 */
+    CU_ASSERT(compare_port_list(in2_pl, acc_pl, 1) == 1);	/* Only match is needed from access port list - 2 */
+    CU_ASSERT(compare_port_list(in1_pl, acc_pl, 0) == 1);	/* All ports must match access port list - 1 */
+    CU_ASSERT(compare_port_list(in2_pl, acc_pl, 0) == 1);	/* All ports must match access port list - 2 */
+    CU_ASSERT(compare_port_list(acc_pl, in1_pl, 0) == 0);	/* All ports must match in1 port list - 1 */
+    CU_ASSERT(compare_port_list(acc_pl, in2_pl, 0) == 1);	/* All ports must match in2 port list - 2 */
+}
+
+int register_ts_access(void)
+{
+    ts_init(&TEST_SUITE(access), TEST_SUITE_DESCR(access), NULL, NULL);
+    ts_add_utest(&TEST_SUITE(access), UTEST_FCT(compare_port_list), UTEST_DESCR(compare_port_list));
+
+    return register_ts(&TEST_SUITE(access));
+}
+#endif /* HAVE_C_UNIT_TESTS */
 
 /***EOF***/
