@@ -359,7 +359,6 @@ validate_options(fko_srv_options_t *opts)
         if(tmp_path[strlen(tmp_path)-1] != '/')
             strlcat(tmp_path, "/", sizeof(tmp_path));
 
-
 #if USE_FILE_CACHE
         strlcat(tmp_path, DEF_DIGEST_CACHE_FILENAME, sizeof(tmp_path));
         set_config_entry(opts, CONF_DIGEST_FILE, tmp_path);
@@ -428,6 +427,12 @@ validate_options(fko_srv_options_t *opts)
     if(opts->config[CONF_ENABLE_DIGEST_PERSISTENCE] == NULL)
         set_config_entry(opts, CONF_ENABLE_DIGEST_PERSISTENCE,
             DEF_ENABLE_DIGEST_PERSISTENCE);
+            
+    /* Enable destination rule.
+    */
+    if(opts->config[CONF_ENABLE_DESTINATION_RULE] == NULL)
+        set_config_entry(opts, CONF_ENABLE_DESTINATION_RULE,
+            DEF_ENABLE_DESTINATION_RULE);
 
     /* Max sniff bytes.
     */
@@ -1037,6 +1042,15 @@ config_init(fko_srv_options_t *opts, int argc, char **argv)
 #else
                 log_msg(LOG_ERR, "[*] fwknopd not compiled with AFL fuzzing support");
                 clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+#endif
+                break;
+            case AFL_PKT_FILE:
+#if AFL_FUZZING
+                opts->afl_fuzzing = 1;
+                set_config_entry(opts, CONF_AFL_PKT_FILE, optarg);
+#else
+                log_msg(LOG_ERR, "[*] fwknopd not compiled with AFL fuzzing support");
+                clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
 #endif
                 break;
             case 'a':
@@ -1143,6 +1157,9 @@ config_init(fko_srv_options_t *opts, int argc, char **argv)
             case 'R':
                 opts->restart = 1;
                 break;
+            case 'r':
+                set_config_entry(opts, CONF_FWKNOP_RUN_DIR, optarg);
+                break;
             case 'S':
                 opts->status = 1;
                 break;
@@ -1225,6 +1242,9 @@ usage(void)
       "                           override the PCAP_FILTER variable in fwknopd.conf.\n"
       " -R, --restart           - Force the currently running fwknopd to restart.\n"
       "     --rotate-digest-cache\n"
+      "                         - Rotate the digest cache file by renaming the file\n"
+      "                           to the same path with the -old suffix.\n"
+      " -r, --run-dir           - Set path to local state run directory.\n"
       "                         - Rotate the digest cache file by renaming it to\n"
       "                           '<name>-old', and starting a new one.\n"
       " -S, --status            - Display the status of any running fwknopd process.\n"
@@ -1235,8 +1255,8 @@ usage(void)
       "     --syslog-enable     - Allow messages to be sent to syslog even if the\n"
       "                           foreground mode is set.\n"
       " -V, --version           - Print version number.\n"
-      " -A, --afl-fuzzing       - Run in American Fuzzy Lop (AFL) fuzzing mode\n"
-      "                           plaintext SPA packets are accepted via stdin.\n"
+      " -A, --afl-fuzzing       - Run in American Fuzzy Lop (AFL) fuzzing mode so\n"
+      "                           that plaintext SPA packets are accepted via stdin.\n"
       " -h, --help              - Print this usage message and exit.\n"
       " --dump-serv-err-codes   - List all server error codes (only needed by the\n"
       "                           test suite).\n"
