@@ -70,7 +70,7 @@ chop_newline(char *str)
 static int
 rule_exists_no_chk_support(const fko_srv_options_t * const opts,
         const struct fw_chain * const fwc, const unsigned int proto,
-        const char * const srcip, const char * const dstip, 
+        const char * const srcip, const char * const dstip,
         const unsigned int port, const unsigned int exp_ts)
 {
     int     rule_exists=0;
@@ -89,8 +89,8 @@ rule_exists_no_chk_support(const fko_srv_options_t * const opts,
      * maximize code coverage in conjunction with the test suite, and is never
      * compiled in for a production release of fwknop.
     */
-    if(run_extcmd_write("/bin/grep -v test", "/bin/echo test", &pid_status, opts) != 0)
-        log_msg(LOG_WARNING, "Code coverage: Could not execute command");
+    if(run_extcmd_write("/bin/grep -v test", "/bin/echo test", &pid_status, opts) == 0)
+        log_msg(LOG_WARNING, "[ignore] Code coverage: Executed command");
 #endif
 
     snprintf(cmd_buf, CMD_BUFSIZE-1, "%s " IPT_LIST_RULES_ARGS,
@@ -122,11 +122,14 @@ rule_exists_no_chk_support(const fko_srv_options_t * const opts,
      * primary search method
     */
     if(search_extcmd(cmd_buf, WANT_STDERR, NO_TIMEOUT, exp_ts_search, &pid_status, opts)
-            && search_extcmd(cmd_buf, WANT_STDERR, NO_TIMEOUT, proto_search, &pid_status, opts)
+            && (proto == ANY_PROTO) ? 1 :
+                search_extcmd(cmd_buf, WANT_STDERR, NO_TIMEOUT, proto_search, &pid_status, opts)
             && search_extcmd(cmd_buf, WANT_STDERR, NO_TIMEOUT, srcip_search, &pid_status, opts)
-            && (dstip == NULL) ? 1 : search_extcmd(cmd_buf, WANT_STDERR, NO_TIMEOUT, dstip_search, &pid_status, opts)
+            && (dstip == NULL) ? 1 :
+                search_extcmd(cmd_buf, WANT_STDERR, NO_TIMEOUT, dstip_search, &pid_status, opts)
             && search_extcmd(cmd_buf, WANT_STDERR, NO_TIMEOUT, target_search, &pid_status, opts)
-            && search_extcmd(cmd_buf, WANT_STDERR, NO_TIMEOUT, port_search, &pid_status, opts))
+            && (port == ANY_PORT) ? 1 :
+                search_extcmd(cmd_buf, WANT_STDERR, NO_TIMEOUT, port_search, &pid_status, opts))
         rule_exists = 1;
 
     if(rule_exists)
@@ -179,7 +182,7 @@ static int
 rule_exists(const fko_srv_options_t * const opts,
         const struct fw_chain * const fwc, const char * const rule,
         const unsigned int proto, const char * const srcip,
-        const char * const dstip, const unsigned int port, 
+        const char * const dstip, const unsigned int port,
         const unsigned int exp_ts)
 {
     int rule_exists = 0;
@@ -187,7 +190,8 @@ rule_exists(const fko_srv_options_t * const opts,
     if(have_ipt_chk_support == 1)
         rule_exists = rule_exists_chk_support(opts, fwc->to_chain, rule);
     else
-        rule_exists = rule_exists_no_chk_support(opts, fwc, proto, srcip, (opts->fw_config->use_destination ? dstip : NULL), port, exp_ts);
+        rule_exists = rule_exists_no_chk_support(opts, fwc, proto, srcip,
+                (opts->fw_config->use_destination ? dstip : NULL), port, exp_ts);
 
     if(rule_exists == 1)
         log_msg(LOG_DEBUG, "rule_exists() Rule : '%s' in %s already exists",
@@ -342,7 +346,7 @@ comment_match_exists(const fko_srv_options_t * const opts)
 static int
 add_jump_rule(const fko_srv_options_t * const opts, const int chain_num)
 {
-    int res = 0;
+    int res = 0, rv = 0;
 
     zero_cmd_buffers();
 
@@ -361,13 +365,16 @@ add_jump_rule(const fko_srv_options_t * const opts, const int chain_num)
         cmd_buf, res, err_buf);
 
     if(EXTCMD_IS_SUCCESS(res))
+    {
         log_msg(LOG_INFO, "Added jump rule from chain: %s to chain: %s",
             fwc.chain[chain_num].from_chain,
             fwc.chain[chain_num].to_chain);
+        rv = 1;
+    }
     else
         log_msg(LOG_ERR, "Error %i from cmd:'%s': %s", res, cmd_buf, err_buf);
 
-    return res;
+    return rv;
 }
 
 static int
@@ -480,9 +487,8 @@ fw_dump_rules(const fko_srv_options_t * const opts)
         fprintf(stdout, "Listing all iptables rules in applicable tables...\n");
         fflush(stdout);
 
-        for(i=0; i<(NUM_FWKNOP_ACCESS_TYPES); i++)
+        for(i=0; i < NUM_FWKNOP_ACCESS_TYPES; i++)
         {
-
             if(fwc.chain[i].target[0] == '\0')
                 continue;
 
@@ -514,9 +520,8 @@ fw_dump_rules(const fko_srv_options_t * const opts)
         fprintf(stdout, "Listing rules in fwknopd iptables chains...\n");
         fflush(stdout);
 
-        for(i=0; i<(NUM_FWKNOP_ACCESS_TYPES); i++)
+        for(i=0; i < NUM_FWKNOP_ACCESS_TYPES; i++)
         {
-
             if(fwc.chain[i].target[0] == '\0')
                 continue;
 
@@ -558,7 +563,7 @@ delete_all_chains(const fko_srv_options_t * const opts)
 {
     int     i, res, cmd_ctr = 0;
 
-    for(i=0; i<(NUM_FWKNOP_ACCESS_TYPES); i++)
+    for(i=0; i < NUM_FWKNOP_ACCESS_TYPES; i++)
     {
         if(fwc.chain[i].target[0] == '\0')
             continue;
@@ -638,7 +643,7 @@ delete_all_chains(const fko_srv_options_t * const opts)
 static int
 create_chain(const fko_srv_options_t * const opts, const int chain_num)
 {
-    int res = 0;
+    int res = 0, rv = 0;
 
     zero_cmd_buffers();
 
@@ -658,10 +663,30 @@ create_chain(const fko_srv_options_t * const opts, const int chain_num)
         cmd_buf, res, err_buf);
 
     /* Expect full success on this */
-    if(! EXTCMD_IS_SUCCESS(res))
+    if(EXTCMD_IS_SUCCESS(res))
+        rv = 1;
+    else
         log_msg(LOG_ERR, "Error %i from cmd:'%s': %s", res, cmd_buf, err_buf);
 
-    return res;
+    return rv;
+}
+
+static int
+mk_chain(const fko_srv_options_t * const opts, const int chain_num)
+{
+    int err = 0;
+
+    /* Make sure the required chain and jump rule exist
+    */
+    if(! chain_exists(opts, chain_num))
+        if(! create_chain(opts, chain_num))
+            err++;
+
+    if (! jump_rule_exists(opts, chain_num))
+        if(! add_jump_rule(opts, chain_num))
+            err++;
+
+    return err;
 }
 
 /* Create the fwknop custom chains (at least those that are configured).
@@ -671,26 +696,12 @@ create_fw_chains(const fko_srv_options_t * const opts)
 {
     int     i, got_err = 0;
 
-    for(i=0; i<(NUM_FWKNOP_ACCESS_TYPES); i++)
+    for(i=0; i < NUM_FWKNOP_ACCESS_TYPES; i++)
     {
         if(fwc.chain[i].target[0] == '\0')
             continue;
 
-        if(chain_exists(opts, i) == 0)
-        {
-
-            /* Create the chain
-            */
-            if(! EXTCMD_IS_SUCCESS(create_chain(opts, i)))
-                got_err++;
-
-            /* Then create the jump rule to that chain if it
-             * doesn't already exist (which is possible)
-            */
-            if(jump_rule_exists(opts, i) == 0)
-                if(! EXTCMD_IS_SUCCESS(add_jump_rule(opts, i)))
-                    got_err++;
-        }
+        got_err += mk_chain(opts, i);
     }
 
     return(got_err);
@@ -794,7 +805,6 @@ set_fw_chain_conf(const int type, const char * const conf_str)
 int
 fw_config_init(fko_srv_options_t * const opts)
 {
-
     memset(&fwc, 0x0, sizeof(struct fw_config));
 
     /* Set our firewall exe command path (iptables in most cases).
@@ -812,7 +822,7 @@ fw_config_init(fko_srv_options_t * const opts)
     if(set_fw_chain_conf(IPT_INPUT_ACCESS, opts->config[CONF_IPT_INPUT_ACCESS]) != 1)
         return 0;
 
-    /* The FWKNOP_OUTPUT_ACCESS requires ENABLE_IPT_OUTPUT_ACCESS be Y
+    /* The FWKNOP_OUTPUT_ACCESS requires ENABLE_IPT_OUTPUT_ACCESS == Y
     */
     if(strncasecmp(opts->config[CONF_ENABLE_IPT_OUTPUT], "Y", 1)==0)
         if(set_fw_chain_conf(IPT_OUTPUT_ACCESS, opts->config[CONF_IPT_OUTPUT_ACCESS]) != 1)
@@ -828,32 +838,23 @@ fw_config_init(fko_srv_options_t * const opts)
         if(set_fw_chain_conf(IPT_DNAT_ACCESS, opts->config[CONF_IPT_DNAT_ACCESS]) != 1)
             return 0;
 
-        /* SNAT (whichever mode) requires ENABLE_IPT_SNAT = Y
+        /* Requires ENABLE_IPT_SNAT = Y
         */
         if(strncasecmp(opts->config[CONF_ENABLE_IPT_SNAT], "Y", 1)==0)
         {
-            if(opts->config[CONF_SNAT_TRANSLATE_IP] == NULL
-                    || ! is_valid_ipv4_addr(opts->config[CONF_SNAT_TRANSLATE_IP]))
-            {
-                fwc.use_masquerade = 1;
-                if(set_fw_chain_conf(IPT_MASQUERADE_ACCESS, opts->config[CONF_IPT_MASQUERADE_ACCESS]) != 1)
-                    return 0;
-            }
-            else
-            {
-                if(is_valid_ipv4_addr(opts->config[CONF_SNAT_TRANSLATE_IP]))
-                {
-                    if(set_fw_chain_conf(IPT_SNAT_ACCESS, opts->config[CONF_IPT_SNAT_ACCESS]) != 1)
-                        return 0;
-                }
-                else
-                {
-                    return 0;
-                }
-            }
+            /* Support both SNAT and MASQUERADE - this will be controlled
+             * via the access.conf configuration for individual rules
+            */
+            if(set_fw_chain_conf(IPT_MASQUERADE_ACCESS,
+                        opts->config[CONF_IPT_MASQUERADE_ACCESS]) != 1)
+                return 0;
+
+            if(set_fw_chain_conf(IPT_SNAT_ACCESS,
+                        opts->config[CONF_IPT_SNAT_ACCESS]) != 1)
+                return 0;
         }
     }
-    
+
     if(strncasecmp(opts->config[CONF_ENABLE_DESTINATION_RULE], "Y", 1)==0)
     {
         fwc.use_destination = 1;
@@ -930,7 +931,8 @@ create_rule(const fko_srv_options_t * const opts,
 
     zero_cmd_buffers();
 
-    snprintf(cmd_buf, CMD_BUFSIZE-1, "%s -A %s %s", opts->fw_config->fw_command, fw_chain, fw_rule);
+    snprintf(cmd_buf, CMD_BUFSIZE-1, "%s -A %s %s",
+            opts->fw_config->fw_command, fw_chain, fw_rule);
 
     res = run_extcmd(cmd_buf, err_buf, CMD_BUFSIZE, WANT_STDERR,
                 NO_TIMEOUT, &pid_status, opts);
@@ -941,7 +943,8 @@ create_rule(const fko_srv_options_t * const opts,
 
     if(EXTCMD_IS_SUCCESS(res))
     {
-        log_msg(LOG_DEBUG, "create_rule() Rule: '%s' added to %s", fw_rule, fw_chain);
+        log_msg(LOG_DEBUG, "create_rule() Rule: '%s' added to %s",
+                fw_rule, fw_chain);
         res = 1;
     }
     else
@@ -950,6 +953,281 @@ create_rule(const fko_srv_options_t * const opts,
     return res;
 }
 
+static void
+ipt_rule(const fko_srv_options_t * const opts,
+        const char * const complete_rule_buf,
+        const char * const fw_rule_macro,
+        const char * const srcip, const char * const dstip,
+        const unsigned int proto, const unsigned int port,
+        struct fw_chain * const chain,
+        const unsigned int exp_ts, const time_t now,
+        const char * const msg, const char * const access_msg)
+{
+    char rule_buf[CMD_BUFSIZE] = {0};
+
+    if(complete_rule_buf != NULL && complete_rule_buf[0] != 0x0)
+    {
+        strlcpy(rule_buf, complete_rule_buf, CMD_BUFSIZE-1);
+    }
+    else
+    {
+        memset(rule_buf, 0, CMD_BUFSIZE);
+
+        snprintf(rule_buf, CMD_BUFSIZE-1, fw_rule_macro,
+            chain->table,
+            proto,
+            srcip,
+            dstip,
+            port,
+            exp_ts,
+            chain->target
+        );
+    }
+
+    /* Check to make sure that the chain and jump rule exists
+    */
+    mk_chain(opts, chain->type);
+
+    if(rule_exists(opts, chain, rule_buf,
+                proto, srcip, dstip, port, exp_ts) == 0)
+    {
+        if(create_rule(opts, chain->to_chain, rule_buf))
+        {
+            log_msg(LOG_INFO, "Added %s rule to %s for %s -> %s %s, expires at %u",
+                msg, chain->to_chain, srcip, (dstip == NULL) ? IPT_ANY_IP : dstip,
+                access_msg, exp_ts
+            );
+
+            chain->active_rules++;
+
+            /* Reset the next expected expire time for this chain if it
+            * is warranted.
+            */
+            if(chain->next_expire < now || exp_ts < chain->next_expire)
+                chain->next_expire = exp_ts;
+        }
+    }
+
+    return;
+}
+
+static void forward_access_rule(const fko_srv_options_t * const opts,
+        const acc_stanza_t * const acc,
+        struct fw_chain * const fwd_chain,
+        const char * const nat_ip,
+        const unsigned int nat_port,
+        const unsigned int fst_proto,
+        const unsigned int fst_port,
+        spa_data_t * const spadat,
+        const unsigned int exp_ts,
+        const time_t now)
+{
+    char   rule_buf[CMD_BUFSIZE] = {0};
+
+    log_msg(LOG_DEBUG,
+            "forward_access_rule() forward_all: %d, nat_ip: %s, nat_port: %d",
+            acc->forward_all, nat_ip, nat_port);
+
+    if(acc->forward_all)
+    {
+        memset(rule_buf, 0, CMD_BUFSIZE);
+
+        snprintf(rule_buf, CMD_BUFSIZE-1, IPT_FWD_ALL_RULE_ARGS,
+            fwd_chain->table,
+            spadat->use_src_ip,
+            exp_ts,
+            fwd_chain->target
+        );
+
+        /* Make a global ACCEPT rule for all ports/protocols
+        */
+        ipt_rule(opts, rule_buf, NULL, spadat->use_src_ip,
+            NULL, ANY_PROTO, ANY_PORT, fwd_chain, exp_ts, now,
+            "FORWARD ALL", "*/*");
+    }
+    else
+    {
+        /* Make the FORWARD access rule
+        */
+        ipt_rule(opts, NULL, IPT_FWD_RULE_ARGS, spadat->use_src_ip,
+            nat_ip, fst_proto, nat_port, fwd_chain, exp_ts, now,
+            "FORWARD", spadat->spa_message_remain);
+
+    }
+    return;
+}
+
+static void dnat_rule(const fko_srv_options_t * const opts,
+        const acc_stanza_t * const acc,
+        struct fw_chain * const dnat_chain,
+        const char * const nat_ip,
+        const unsigned int nat_port,
+        const unsigned int fst_proto,
+        const unsigned int fst_port,
+        spa_data_t * const spadat,
+        const unsigned int exp_ts,
+        const time_t now)
+{
+    char   rule_buf[CMD_BUFSIZE] = {0};
+
+    log_msg(LOG_DEBUG, "dnat_rule() forward_all: %d, nat_ip: %s, nat_port: %d",
+            acc->forward_all, nat_ip, nat_port);
+
+    if(acc->forward_all)
+    {
+        memset(rule_buf, 0, CMD_BUFSIZE);
+
+        snprintf(rule_buf, CMD_BUFSIZE-1, IPT_DNAT_ALL_RULE_ARGS,
+            dnat_chain->table,
+            spadat->use_src_ip,
+            (fwc.use_destination ? spadat->pkt_destination_ip : IPT_ANY_IP),
+            exp_ts,
+            dnat_chain->target,
+            nat_ip
+        );
+
+        /* Make a global DNAT rule for all ports/protocols
+        */
+        ipt_rule(opts, rule_buf, NULL, spadat->use_src_ip,
+            NULL, ANY_PROTO, ANY_PORT, dnat_chain, exp_ts, now,
+            "DNAT ALL", "*/*");
+    }
+    else
+    {
+        memset(rule_buf, 0, CMD_BUFSIZE);
+
+        snprintf(rule_buf, CMD_BUFSIZE-1, IPT_DNAT_RULE_ARGS,
+            dnat_chain->table,
+            fst_proto,
+            spadat->use_src_ip,
+            (fwc.use_destination ? spadat->pkt_destination_ip : IPT_ANY_IP),
+            fst_port,
+            exp_ts,
+            dnat_chain->target,
+            nat_ip,
+            nat_port
+        );
+
+        ipt_rule(opts, rule_buf, NULL, spadat->use_src_ip,
+            (fwc.use_destination ? spadat->pkt_destination_ip : IPT_ANY_IP),
+            fst_proto, fst_port, dnat_chain, exp_ts, now, "DNAT",
+            spadat->spa_message_remain);
+    }
+    return;
+}
+
+static void snat_rule(const fko_srv_options_t * const opts,
+        const acc_stanza_t * const acc,
+        const char * const nat_ip,
+        const unsigned int nat_port,
+        const unsigned int fst_proto,
+        const unsigned int fst_port,
+        spa_data_t * const spadat,
+        const unsigned int exp_ts,
+        const time_t now)
+{
+    char     rule_buf[CMD_BUFSIZE] = {0};
+    char     snat_target[SNAT_TARGET_BUFSIZE] = {0};
+    struct   fw_chain *snat_chain = NULL;
+
+    log_msg(LOG_DEBUG,
+            "snat_rule() forward_all: %d, nat_ip: %s, nat_port: %d, force_snat: %d, force_snat_ip: %s, force_masq: %d",
+            acc->forward_all, nat_ip, nat_port, acc->force_snat,
+            (acc->force_snat_ip == NULL) ? "(NONE)" : acc->force_snat_ip,
+            acc->force_masquerade);
+
+    if(acc->forward_all)
+    {
+        /* Default to MASQUERADE */
+        snat_chain = &(opts->fw_config->chain[IPT_MASQUERADE_ACCESS]);
+        snprintf(snat_target, SNAT_TARGET_BUFSIZE-1, " ");
+
+        /* Add SNAT or MASQUERADE rules.
+        */
+        if(acc->force_snat && is_valid_ipv4_addr(acc->force_snat_ip))
+        {
+            /* Using static SNAT */
+            snat_chain = &(opts->fw_config->chain[IPT_SNAT_ACCESS]);
+            snprintf(snat_target, SNAT_TARGET_BUFSIZE-1,
+                "--to-source %s", acc->force_snat_ip);
+        }
+        else if((opts->config[CONF_SNAT_TRANSLATE_IP] != NULL)
+            && is_valid_ipv4_addr(opts->config[CONF_SNAT_TRANSLATE_IP]))
+        {
+            /* Using static SNAT */
+            snat_chain = &(opts->fw_config->chain[IPT_SNAT_ACCESS]);
+            snprintf(snat_target, SNAT_TARGET_BUFSIZE-1,
+                "--to-source %s", opts->config[CONF_SNAT_TRANSLATE_IP]);
+        }
+
+        memset(rule_buf, 0, CMD_BUFSIZE);
+
+        snprintf(rule_buf, CMD_BUFSIZE-1, IPT_SNAT_ALL_RULE_ARGS,
+            snat_chain->table,
+            spadat->use_src_ip,
+            exp_ts,
+            snat_chain->target,
+            snat_target
+        );
+
+        ipt_rule(opts, rule_buf, NULL, spadat->use_src_ip,
+            NULL, ANY_PROTO, ANY_PORT, snat_chain, exp_ts, now,
+            "SNAT ALL", "*/*");
+    }
+    else
+    {
+        /* Add SNAT or MASQUERADE rules.
+        */
+        if(acc->force_snat && is_valid_ipv4_addr(acc->force_snat_ip))
+        {
+            /* Using static SNAT */
+            snat_chain = &(opts->fw_config->chain[IPT_SNAT_ACCESS]);
+            snprintf(snat_target, SNAT_TARGET_BUFSIZE-1,
+                "--to-source %s:%i", acc->force_snat_ip, fst_port);
+        }
+        else if(acc->force_snat && acc->force_masquerade)
+        {
+            /* Using MASQUERADE */
+            snat_chain = &(opts->fw_config->chain[IPT_MASQUERADE_ACCESS]);
+            snprintf(snat_target, SNAT_TARGET_BUFSIZE-1,
+                "--to-ports %i", fst_port);
+        }
+        else if((opts->config[CONF_SNAT_TRANSLATE_IP] != NULL)
+            && is_valid_ipv4_addr(opts->config[CONF_SNAT_TRANSLATE_IP]))
+        {
+            /* Using static SNAT */
+            snat_chain = &(opts->fw_config->chain[IPT_SNAT_ACCESS]);
+            snprintf(snat_target, SNAT_TARGET_BUFSIZE-1,
+                "--to-source %s:%i", opts->config[CONF_SNAT_TRANSLATE_IP],
+                fst_port);
+        }
+        else
+        {
+            /* Using MASQUERADE */
+            snat_chain = &(opts->fw_config->chain[IPT_MASQUERADE_ACCESS]);
+            snprintf(snat_target, SNAT_TARGET_BUFSIZE-1,
+                "--to-ports %i", fst_port);
+        }
+
+        memset(rule_buf, 0, CMD_BUFSIZE);
+
+        snprintf(rule_buf, CMD_BUFSIZE-1, IPT_SNAT_RULE_ARGS,
+            snat_chain->table,
+            fst_proto,
+            nat_ip,
+            nat_port,
+            exp_ts,
+            snat_chain->target,
+            snat_target
+        );
+
+        ipt_rule(opts, rule_buf, NULL, spadat->use_src_ip,
+                NULL, fst_proto, nat_port, snat_chain, exp_ts, now, "SNAT",
+                spadat->spa_message_remain);
+    }
+    return;
+}
+
 /****************************************************************************/
 
 /* Rule Processing - Create an access request...
@@ -958,16 +1236,8 @@ int
 process_spa_request(const fko_srv_options_t * const opts,
         const acc_stanza_t * const acc, spa_data_t * const spadat)
 {
-    char             nat_ip[MAX_IPV4_STR_LEN] = {0};
-    char             snat_target[SNAT_TARGET_BUFSIZE] = {0};
-    char             rule_buf[CMD_BUFSIZE] = {0};
-    char            *ndx;
-
-    unsigned int     nat_port = 0;
-
-    acc_port_list_t *port_list = NULL;
-    acc_port_list_t *ple = NULL;
-
+    char            nat_ip[MAX_IPV4_STR_LEN] = {0};
+    unsigned int    nat_port = 0;
     unsigned int    fst_proto;
     unsigned int    fst_port;
 
@@ -975,9 +1245,12 @@ process_spa_request(const fko_srv_options_t * const opts,
     struct fw_chain * const out_chain  = &(opts->fw_config->chain[IPT_OUTPUT_ACCESS]);
     struct fw_chain * const fwd_chain  = &(opts->fw_config->chain[IPT_FORWARD_ACCESS]);
     struct fw_chain * const dnat_chain = &(opts->fw_config->chain[IPT_DNAT_ACCESS]);
-    struct fw_chain *snat_chain; /* We assign this later (if we need to). */
 
-    int             res = 0, is_err, snat_chain_num = 0;
+    acc_port_list_t *port_list = NULL;
+    acc_port_list_t *ple = NULL;
+
+    char            *ndx = NULL;
+    int             res = 0, is_err;
     time_t          now;
     unsigned int    exp_ts;
 
@@ -1008,115 +1281,14 @@ process_spa_request(const fko_srv_options_t * const opts,
     time(&now);
     exp_ts = now + spadat->fw_access_timeout;
 
-    /* For straight access requests, we currently support multiple proto/port
-     * request.
+    /* deal with SPA packets that themselves request a NAT operation
     */
-    if((spadat->message_type == FKO_ACCESS_MSG
-      || spadat->message_type == FKO_CLIENT_TIMEOUT_ACCESS_MSG) && !acc->force_nat)
-    {
-
-        /* Check to make sure that the jump rules exist for each
-         * required chain
-        */
-        if(chain_exists(opts, IPT_INPUT_ACCESS) == 0)
-            create_chain(opts, IPT_INPUT_ACCESS);
-
-        if(jump_rule_exists(opts, IPT_INPUT_ACCESS) == 0)
-            add_jump_rule(opts, IPT_INPUT_ACCESS);
-
-        if(strlen(out_chain->to_chain))
-        {
-            if(chain_exists(opts, IPT_OUTPUT_ACCESS) == 0)
-                create_chain(opts, IPT_OUTPUT_ACCESS);
-
-            if(jump_rule_exists(opts, IPT_OUTPUT_ACCESS) == 0)
-                add_jump_rule(opts, IPT_OUTPUT_ACCESS);
-        }
-
-        /* Create an access command for each proto/port for the source ip.
-        */
-        while(ple != NULL)
-        {
-            memset(rule_buf, 0, CMD_BUFSIZE);
-
-            snprintf(rule_buf, CMD_BUFSIZE-1, IPT_RULE_ARGS,
-                in_chain->table,
-                ple->proto,
-                spadat->use_src_ip,
-                (fwc.use_destination ? spadat->pkt_destination_ip : IPT_ANY_IP),
-                ple->port,
-                exp_ts,
-                in_chain->target
-            );
-
-            if(rule_exists(opts, in_chain, rule_buf,
-                        ple->proto, spadat->use_src_ip, spadat->pkt_destination_ip, ple->port, exp_ts) == 0)
-            {
-                if(create_rule(opts, in_chain->to_chain, rule_buf))
-                {
-                    log_msg(LOG_INFO, "Added Rule to %s for %s, %s expires at %u",
-                        in_chain->to_chain, spadat->use_src_ip,
-                        spadat->spa_message_remain, exp_ts
-                    );
-
-                    in_chain->active_rules++;
-
-                    /* Reset the next expected expire time for this chain if it
-                    * is warranted.
-                    */
-                    if(in_chain->next_expire < now || exp_ts < in_chain->next_expire)
-                        in_chain->next_expire = exp_ts;
-                }
-            }
-
-            /* If we have to make an corresponding OUTPUT rule if out_chain target
-            * is not NULL.
-            */
-            if(strlen(out_chain->to_chain))
-            {
-                memset(rule_buf, 0, CMD_BUFSIZE);
-
-                snprintf(rule_buf, CMD_BUFSIZE-1, IPT_OUT_RULE_ARGS,
-                    out_chain->table,
-                    ple->proto,
-                    spadat->use_src_ip,
-                    (fwc.use_destination ? spadat->pkt_destination_ip : IPT_ANY_IP),
-                    ple->port,
-                    exp_ts,
-                    out_chain->target
-                );
-
-                if(rule_exists(opts, out_chain, rule_buf,
-                        ple->proto, spadat->use_src_ip, spadat->pkt_destination_ip, ple->port, exp_ts) == 0)
-                {
-                    if(create_rule(opts, out_chain->to_chain, rule_buf))
-                    {
-                        log_msg(LOG_INFO, "Added Rule in %s for %s, %s expires at %u",
-                            out_chain->to_chain, spadat->use_src_ip,
-                            spadat->spa_message_remain, exp_ts
-                        );
-
-                        out_chain->active_rules++;
-
-                        /* Reset the next expected expire time for this chain if it
-                        * is warranted.  */
-                        if(out_chain->next_expire < now || exp_ts < out_chain->next_expire)
-                            out_chain->next_expire = exp_ts;
-                    }
-                }
-            }
-            ple = ple->next;
-        }
-    }
-    /* NAT requests... */
-    else if(spadat->message_type == FKO_LOCAL_NAT_ACCESS_MSG
+    if(spadat->message_type == FKO_LOCAL_NAT_ACCESS_MSG
       || spadat->message_type == FKO_CLIENT_TIMEOUT_LOCAL_NAT_ACCESS_MSG
       || spadat->message_type == FKO_NAT_ACCESS_MSG
       || spadat->message_type == FKO_CLIENT_TIMEOUT_NAT_ACCESS_MSG
       || acc->force_nat)
     {
-        /* Parse out the NAT IP and Port components.
-        */
         if(acc->force_nat)
         {
             strlcpy(nat_ip, acc->force_nat_ip, sizeof(nat_ip));
@@ -1135,7 +1307,8 @@ process_spa_request(const fko_srv_options_t * const opts,
                     return res;
                 }
 
-                nat_port = strtol_wrapper(ndx+1, 0, MAX_PORT, NO_EXIT_UPON_ERR, &is_err);
+                nat_port = strtol_wrapper(ndx+1, 0, MAX_PORT,
+                        NO_EXIT_UPON_ERR, &is_err);
                 if(is_err != FKO_SUCCESS)
                 {
                     log_msg(LOG_INFO, "Invalid NAT port in SPA message");
@@ -1148,214 +1321,53 @@ process_spa_request(const fko_srv_options_t * const opts,
 
         if(spadat->message_type == FKO_LOCAL_NAT_ACCESS_MSG)
         {
-            memset(rule_buf, 0, CMD_BUFSIZE);
-
-            snprintf(rule_buf, CMD_BUFSIZE-1, IPT_RULE_ARGS,
-                in_chain->table,
-                fst_proto,
-                spadat->use_src_ip,
+            ipt_rule(opts, NULL, IPT_RULE_ARGS, spadat->use_src_ip,
                 (fwc.use_destination ? spadat->pkt_destination_ip : IPT_ANY_IP),
-                nat_port,
-                exp_ts,
-                in_chain->target
-            );
-
-            /* Check to make sure that the jump rules exist for each
-             * required chain
-            */
-            if(chain_exists(opts, IPT_INPUT_ACCESS) == 0)
-                create_chain(opts, IPT_INPUT_ACCESS);
-
-            if(jump_rule_exists(opts, IPT_INPUT_ACCESS) == 0)
-                add_jump_rule(opts, IPT_INPUT_ACCESS);
-
-            if(rule_exists(opts, in_chain, rule_buf,
-                        fst_proto, spadat->use_src_ip, spadat->pkt_destination_ip, nat_port, exp_ts) == 0)
-            {
-                if(create_rule(opts, in_chain->to_chain, rule_buf))
-                {
-                    log_msg(LOG_INFO, "Added Rule to %s for %s, %s expires at %u",
-                        in_chain->to_chain, spadat->use_src_ip,
-                        spadat->spa_message_remain, exp_ts
-                    );
-
-                    in_chain->active_rules++;
-
-                    /* Reset the next expected expire time for this chain if it
-                    * is warranted.
-                    */
-                    if(in_chain->next_expire < now || exp_ts < in_chain->next_expire)
-                        in_chain->next_expire = exp_ts;
-                }
-            }
+                fst_proto, nat_port, in_chain, exp_ts, now, "local NAT",
+                spadat->spa_message_remain);
         }
         else if(strlen(fwd_chain->to_chain))
         {
-            /* Make our FORWARD and NAT rules, and make sure the
-             * required chain and jump rule exists
+            /* FORWARD access rule
             */
-            if(chain_exists(opts, IPT_FORWARD_ACCESS) == 0)
-                create_chain(opts, IPT_FORWARD_ACCESS);
-
-            if (jump_rule_exists(opts, IPT_FORWARD_ACCESS) == 0)
-                add_jump_rule(opts, IPT_FORWARD_ACCESS);
-
-            memset(rule_buf, 0, CMD_BUFSIZE);
-
-            snprintf(rule_buf, CMD_BUFSIZE-1, IPT_FWD_RULE_ARGS,
-                fwd_chain->table,
-                fst_proto,
-                spadat->use_src_ip,
-                nat_ip,
-                nat_port,
-                exp_ts,
-                fwd_chain->target
-            );
-
-            if(rule_exists(opts, fwd_chain, rule_buf, fst_proto,
-                    spadat->use_src_ip, nat_ip, nat_port, exp_ts) == 0)
-            {
-                if(create_rule(opts, fwd_chain->to_chain, rule_buf))
-                {
-                    log_msg(LOG_INFO, "Added FORWARD Rule to %s for %s, %s expires at %u",
-                        fwd_chain->to_chain, spadat->use_src_ip,
-                        spadat->spa_message_remain, exp_ts
-                    );
-
-                    fwd_chain->active_rules++;
-
-                    /* Reset the next expected expire time for this chain if it
-                    * is warranted.
-                    */
-                    if(fwd_chain->next_expire < now || exp_ts < fwd_chain->next_expire)
-                        fwd_chain->next_expire = exp_ts;
-                }
-            }
+            forward_access_rule(opts, acc, fwd_chain, nat_ip,
+                    nat_port, fst_proto, fst_port, spadat, exp_ts, now);
         }
 
-        if(strlen(dnat_chain->to_chain))
-        {
-            /* Make sure the required chain and jump rule exist
-            */
-            if(chain_exists(opts, IPT_DNAT_ACCESS) == 0)
-                create_chain(opts, IPT_DNAT_ACCESS);
+        /* DNAT rule
+        */
+        if(strlen(dnat_chain->to_chain) && !acc->disable_dnat)
+            dnat_rule(opts, acc, dnat_chain, nat_ip,
+                    nat_port, fst_proto, fst_port, spadat, exp_ts, now);
 
-            if (jump_rule_exists(opts, IPT_DNAT_ACCESS) == 0)
-                add_jump_rule(opts, IPT_DNAT_ACCESS);
-
-            memset(rule_buf, 0, CMD_BUFSIZE);
-
-            snprintf(rule_buf, CMD_BUFSIZE-1, IPT_DNAT_RULE_ARGS,
-                dnat_chain->table,
-                fst_proto,
-                spadat->use_src_ip,
-                (fwc.use_destination ? spadat->pkt_destination_ip : IPT_ANY_IP),
-                fst_port,
-                exp_ts,
-                dnat_chain->target,
-                nat_ip,
-                nat_port
-            );
-
-            if(rule_exists(opts, dnat_chain, rule_buf, fst_proto,
-                        spadat->use_src_ip, spadat->pkt_destination_ip, fst_port, exp_ts) == 0)
-            {
-                if(create_rule(opts, dnat_chain->to_chain, rule_buf))
-                {
-                    log_msg(LOG_INFO, "Added DNAT Rule to %s for %s, %s expires at %u",
-                        dnat_chain->to_chain, spadat->use_src_ip,
-                        spadat->spa_message_remain, exp_ts
-                    );
-
-                    dnat_chain->active_rules++;
-
-                    /* Reset the next expected expire time for this chain if it
-                    * is warranted.
-                    */
-                    if(dnat_chain->next_expire < now || exp_ts < dnat_chain->next_expire)
-                        dnat_chain->next_expire = exp_ts;
-                }
-            }
-        }
-
-        /* If SNAT (or MASQUERADE) is wanted, then we add those rules here as well.
+        /* SNAT rule
         */
         if(acc->force_snat || strncasecmp(opts->config[CONF_ENABLE_IPT_SNAT], "Y", 1) == 0)
+            snat_rule(opts, acc, nat_ip, nat_port,
+                    fst_proto, fst_port, spadat, exp_ts, now);
+    }
+    else /* Non-NAT request - this is the typical case. */
+    {
+        /* Create an access command for each proto/port for the source ip.
+        */
+        while(ple != NULL)
         {
-            /* Add SNAT or MASQUERADE rules.
+            ipt_rule(opts, NULL, IPT_RULE_ARGS, spadat->use_src_ip,
+                (fwc.use_destination ? spadat->pkt_destination_ip : IPT_ANY_IP),
+                ple->proto, ple->port, in_chain, exp_ts, now, "access",
+                spadat->spa_message_remain);
+
+            /* We need to make a corresponding OUTPUT rule if out_chain target
+             * is not NULL.
             */
-            if(acc->force_snat && is_valid_ipv4_addr(acc->force_snat_ip))
+            if(strlen(out_chain->to_chain))
             {
-                /* Using static SNAT */
-                snat_chain = &(opts->fw_config->chain[IPT_SNAT_ACCESS]);
-                snprintf(snat_target, SNAT_TARGET_BUFSIZE-1,
-                    "--to-source %s:%i", acc->force_snat_ip, fst_port);
-                snat_chain_num = IPT_SNAT_ACCESS;
-            }
-            else if(acc->force_snat && acc->force_masquerade)
-            {
-                /* Using MASQUERADE */
-                snat_chain = &(opts->fw_config->chain[IPT_MASQUERADE_ACCESS]);
-                snprintf(snat_target, SNAT_TARGET_BUFSIZE-1,
-                    "--to-ports %i", fst_port);
-                snat_chain_num = IPT_MASQUERADE_ACCESS;
-            }
-            else if((opts->config[CONF_SNAT_TRANSLATE_IP] != NULL)
-                && is_valid_ipv4_addr(opts->config[CONF_SNAT_TRANSLATE_IP]))
-            {
-                /* Using static SNAT */
-                snat_chain = &(opts->fw_config->chain[IPT_SNAT_ACCESS]);
-                snprintf(snat_target, SNAT_TARGET_BUFSIZE-1,
-                    "--to-source %s:%i", opts->config[CONF_SNAT_TRANSLATE_IP],
-                    fst_port);
-                snat_chain_num = IPT_SNAT_ACCESS;
-            }
-            else
-            {
-                /* Using MASQUERADE */
-                snat_chain = &(opts->fw_config->chain[IPT_MASQUERADE_ACCESS]);
-                snprintf(snat_target, SNAT_TARGET_BUFSIZE-1,
-                    "--to-ports %i", fst_port);
-                snat_chain_num = IPT_MASQUERADE_ACCESS;
-            }
-
-            if(chain_exists(opts, snat_chain_num) == 0)
-                create_chain(opts, snat_chain_num);
-
-            if(jump_rule_exists(opts, snat_chain_num) == 0)
-                add_jump_rule(opts, snat_chain_num);
-
-            memset(rule_buf, 0, CMD_BUFSIZE);
-
-            snprintf(rule_buf, CMD_BUFSIZE-1, IPT_SNAT_RULE_ARGS,
-                snat_chain->table,
-                fst_proto,
-                nat_ip,
-                nat_port,
-                exp_ts,
-                snat_chain->target,
-                snat_target
-            );
-
-            if(rule_exists(opts, snat_chain, rule_buf, fst_proto,
-                        spadat->use_src_ip, NULL, nat_port, exp_ts) == 0)
-            {
-                if(create_rule(opts, snat_chain->to_chain, rule_buf))
-                {
-                    log_msg(LOG_INFO, "Added SNAT Rule to %s for %s, %s expires at %u",
-                        snat_chain->to_chain, spadat->use_src_ip,
-                        spadat->spa_message_remain, exp_ts
-                    );
-
-                    snat_chain->active_rules++;
-
-                    /* Reset the next expected expire time for this chain if it
-                    * is warranted.
-                    */
-                    if(snat_chain->next_expire < now || exp_ts < snat_chain->next_expire)
-                            snat_chain->next_expire = exp_ts;
-                }
+                ipt_rule(opts, NULL, IPT_OUT_RULE_ARGS, spadat->use_src_ip,
+                    (fwc.use_destination ? spadat->pkt_destination_ip : IPT_ANY_IP),
+                    ple->proto, ple->port, out_chain, exp_ts, now, "OUTPUT",
+                    spadat->spa_message_remain);
             }
+            ple = ple->next;
         }
     }
 
