@@ -70,7 +70,7 @@
   /* Our default run directory is based on LOCALSTATEDIR as set by the
    * configure script. This is where we put the PID and digest cache files.
   */
-  #define DEF_RUN_DIR       SYSRUNDIR"/run/"PACKAGE_NAME
+  #define DEF_RUN_DIR       SYSRUNDIR"/"PACKAGE_NAME
 #endif
 
 /* More Conf defaults
@@ -141,7 +141,7 @@
   #define DEF_FIREWD_FORWARD_ACCESS        "ACCEPT, filter, FORWARD, 1, FWKNOP_FORWARD, 1"
   #define DEF_FIREWD_DNAT_ACCESS           "DNAT, nat, PREROUTING, 1, FWKNOP_PREROUTING, 1"
   #define DEF_FIREWD_SNAT_ACCESS           "SNAT, nat, POSTROUTING, 1, FWKNOP_POSTROUTING, 1"
-  #define DEF_FIREWD_MASQUERADE_ACCESS     "MASQUERADE, nat, POSTROUTING, 1, FWKNOP_POSTROUTING, 1"
+  #define DEF_FIREWD_MASQUERADE_ACCESS     "MASQUERADE, nat, POSTROUTING, 1, FWKNOP_MASQUERADE, 1"
 
   #define RCHK_MAX_FIREWD_RULE_NUM         (2 << 15)
 
@@ -161,7 +161,7 @@
   #define DEF_IPT_FORWARD_ACCESS        "ACCEPT, filter, FORWARD, 1, FWKNOP_FORWARD, 1"
   #define DEF_IPT_DNAT_ACCESS           "DNAT, nat, PREROUTING, 1, FWKNOP_PREROUTING, 1"
   #define DEF_IPT_SNAT_ACCESS           "SNAT, nat, POSTROUTING, 1, FWKNOP_POSTROUTING, 1"
-  #define DEF_IPT_MASQUERADE_ACCESS     "MASQUERADE, nat, POSTROUTING, 1, FWKNOP_POSTROUTING, 1"
+  #define DEF_IPT_MASQUERADE_ACCESS     "MASQUERADE, nat, POSTROUTING, 1, FWKNOP_MASQUERADE, 1"
 
   #define RCHK_MAX_IPT_RULE_NUM         (2 << 15)
 
@@ -389,15 +389,14 @@ typedef struct acc_stanza
     int                  expired;
     int                  encryption_mode;
 
-    /* DNAT parameters
+    /* NAT parameters
     */
     unsigned char        force_nat;
     char                *force_nat_ip;
     char                *force_nat_proto;
     unsigned int         force_nat_port;
-
-    /* SNAT parameters
-    */
+    unsigned char        forward_all;
+    unsigned char        disable_dnat;
     unsigned char        force_snat;
     char                *force_snat_ip;
     unsigned char        force_masquerade;
@@ -450,9 +449,6 @@ typedef struct acc_stanza
       struct fw_chain chain[NUM_FWKNOP_ACCESS_TYPES];
       char            fw_command[MAX_PATH_LEN];
 
-      /* Flag for firewalld SNAT vs. MASQUERADE usage
-      */
-      unsigned char   use_masquerade;
       /* Flag for setting destination field in rule
       */
       unsigned char   use_destination;
@@ -500,9 +496,6 @@ typedef struct acc_stanza
       struct fw_chain chain[NUM_FWKNOP_ACCESS_TYPES];
       char            fw_command[MAX_PATH_LEN];
 
-      /* Flag for iptables SNAT vs. MASQUERADE usage
-      */
-      unsigned char   use_masquerade;
       /* Flag for setting destination field in rule
       */
       unsigned char   use_destination;
