@@ -5,6 +5,6 @@
 cd ../../
 CC=afl-gcc ./extras/apparmor/configure_args.sh $@
 make clean
-make
+AFL_HARDEN=1 make
 cd test/afl
 exit $?
