@@ -10,6 +10,7 @@ FDIR="server-access.out"
 OUT_DIR="$TOP_DIR/$FDIR"
 PREV_OUT_DIR=''
 IN_DIR="test-cases/server-access"
+FUZZ_FILE=$OUT_DIR/afl_access.conf
 
 ### build up our afl-fuzz text banner
 TSTR="fwknopd,access.conf"
@@ -32,9 +33,9 @@ fi
 ### run afl-fuzz
 LD_LIBRARY_PATH=$LIB_DIR afl-fuzz \
     -T $BANNER -t 1000 -i $IN_DIR \
-    -o $OUT_DIR -f $OUT_DIR/afl_access.conf $SERVER \
+    -o $OUT_DIR -f $FUZZ_FILE $SERVER \
     -c ../conf/ipt_snat_fwknopd.conf \
-    -a $OUT_DIR/afl_access.conf \
+    -a $FUZZ_FILE \
     -O ../conf/override_no_digest_tracking_fwknopd.conf \
     -A -f -t --exit-parse-config -v -v -v -r `pwd`/run
 
